use peach_stats::{stats, stats::LoadAverage};

/// System statistics data.
pub struct StatusContext {
    pub cpu_usage_percent: Option<f32>,
    pub disk_usage_percent: Option<u32>,
    pub disk_free: Option<u64>,
    pub load_average: Option<LoadAverage>,
    pub mem_usage_percent: Option<u64>,
    pub mem_used: Option<u64>,
    pub mem_free: Option<u64>,
    pub mem_total: Option<u64>,
    pub uptime: Option<u32>,
}

impl StatusContext {
    pub fn build() -> StatusContext {
        // convert result to Option<CpuStatPercentages>, discard any error
        let cpu_usage_percent = stats::cpu_stats_percent()
            .ok()
            .map(|cpu| (cpu.nice + cpu.system + cpu.user).round());

        let load_average = stats::load_average().ok();

        let mem_stats = stats::mem_stats().ok();
        let (mem_usage_percent, mem_used, mem_free, mem_total) = match mem_stats {
            Some(mem) => (
                Some((mem.used / mem.total) * 100),
                Some(mem.used / 1024),
                Some(mem.free / 1024),
                Some(mem.total / 1024),
            ),
            None => (None, None, None, None),
        };

        let uptime = match stats::uptime() {
            Ok(secs) => {
                let uptime_mins = secs / 60;
                uptime_mins.to_string()
            }
            Err(_) => "Unavailable".to_string(),
        };

        // parse the uptime string to a signed integer (for math)
        let uptime_parsed = uptime.parse::<u32>().ok();

        let disk_usage_stats = match stats::disk_usage() {
            Ok(disks) => disks,
            Err(_) => Vec::new(),
        };

        // select only the partition we're interested in: "/"
        let disk_stats = disk_usage_stats.iter().find(|disk| disk.mountpoint == "/");

        let (disk_usage_percent, disk_free) = match disk_stats {
            Some(disk) => (
                Some(disk.used_percentage),
                // calculate free disk space in megabytes
                Some(disk.one_k_blocks_free / 1024),
            ),
            None => (None, None),
        };

        StatusContext {
            cpu_usage_percent,
            disk_usage_percent,
            disk_free,
            load_average,
            mem_usage_percent,
            mem_used,
            mem_free,
            mem_total,
            uptime: uptime_parsed,
        }
    }
}
