use maud::{html, PreEscaped};

use crate::context::{
    network,
    network::{ConfigureDNSContext, NetworkDetailContext, NetworkListContext},
};
use crate::templates;

// /settings/network/wifi/add
pub fn add_ap(
    ssid: Option<String>,
    flash_msg: Option<String>,
    flash_name: Option<String>,
) -> PreEscaped<String> {
    let back = "/settings/network".to_string();
    let title = "Add WiFi Network".to_string();

    let content = html! {
        (PreEscaped("<!-- NETWORK ADD CREDENTIALS FORM -->"))
        div class="card center" {
            div class="card-container" {
                form id="wifiCreds" action="/settings/network/wifi/add" method="post" {
                    (PreEscaped("<!-- input for network ssid -->"))
                    input id="ssid" name="ssid" class="center input" type="text" placeholder="SSID" title="Network name (SSID) for WiFi access point" value=[ssid] autofocus;
                    (PreEscaped("<!-- input for network password -->"))
                    input id="pass" name="pass" class="center input" type="password" placeholder="Password" title="Password for WiFi access point";
                    div id="buttonDiv" {
                        input id="addWifi" class="button button-primary center" title="Add" type="submit" value="Add";
                        a class="button button-secondary center" href="/settings/network" title="Cancel" { "Cancel" }
                    }
                }
                (PreEscaped("<!-- FLASH MESSAGE -->"))
                (templates::snippets::flash_message(flash_msg, flash_name))
            }
        }
    };

    templates::base::base(back, title, content)
}

/* TODO: I JUST OVERWROTE THE network_detail FUNCTION :'( :'( :'( */
// /settings/network/wifi
pub fn network_detail(
    // the ssid of the network we wish to examine in detail
    selected: String,
    flash_msg: Option<String>,
    flash_name: Option<String>,
) -> PreEscaped<String> {
    // retrieve network detail data
    let context = NetworkDetailContext::build();
    // have credentials for the access point we're viewing previously been saved?
    // ie. is this a known access point?
    let selected_is_saved = context.saved_aps.contains(&selected);

    let back = "/settings/network".to_string();
    let title = "WiFi Network Detail".to_string();

    let content = html! {
        (PreEscaped("<!-- NETWORK DETAIL -->"))
        // select only the access point we are interested in
        @match context.wlan_networks.get_key_value(&selected) {
            Some((ssid, ap)) => {
                @let capsule_class = if ssid == &context.wlan_ssid {
                    "two-grid capsule success-border"
                } else {
                    "two-grid capsule"
                };
                @let ap_status = if ssid == &context.wlan_ssid {
                    "CONNECTED"
                } else if ap.state == "Available" {
                    "AVAILABLE"
                } else {
                    "NOT IN RANGE"
                };
                @let ap_protocol = match &ap.detail {
                    Some(scan) => scan.protocol.clone(),
                    None => "Unknown".to_string()
                };
                @let ap_signal = match ap.signal {
                    Some(signal) => signal.to_string(),
                    None => "Unknown".to_string()
                };
                (PreEscaped("<!-- NETWORK CARD -->"))
                div class="card center" {
                    (PreEscaped("<!-- NETWORK INFO BOX -->"))
                    div class=(capsule_class) title="PeachCloud network mode and status" {
                        (PreEscaped("<!-- left column -->"))
                        (PreEscaped("<!-- NETWORK STATUS ICON -->"))
                        div class="grid-column-1" {
                            img id="wifiIcon" class="center icon" src="/icons/wifi.svg" alt="WiFi icon";
                            label class="center label-small font-gray" for="wifiIcon" title="Access Point Status" { (ap_status) }
                        }
                        (PreEscaped("<!-- right column -->"))
                        (PreEscaped("<!-- NETWORK DETAILED INFO -->"))
                        div class="grid-column-2" {
                            label class="label-small font-gray" for="netSsid" title="WiFi network SSID" { "SSID" }
                            p id="netSsid" class="card-text" title="SSID" { (ssid) }
                            label class="label-small font-gray" for="netSec" title="Security protocol" { "SECURITY" }
                            p id="netSec" class="card-text" title="Security protocol in use by "{(ssid)}"" { (ap_protocol) }
                            label class="label-small font-gray" for="netSig" title="Signal Strength" { "SIGNAL" }
                            p id="netSig" class="card-text" title="Signal strength of WiFi access point" { (ap_signal) }
                        }
                    }
                    (PreEscaped("<!-- BUTTONS -->"))
                    div class="card-container" style="padding-top: 0;" {
                        div id="buttonDiv" {
                            @if context.wlan_ssid == selected {
                                form id="wifiDisconnect" action="/settings/network/wifi/disconnect" method="post" {
                                    // hidden element: allows ssid to be sent in request
                                    input id="disconnectSsid" name="ssid" type="text" value=(ssid) style="display: none;";
                                    input id="disconnectWifi" class="button button-warning center" title="Disconnect from Network" type="submit" value="Disconnect";
                                }
                            }
                            // If the selected access point appears in the list,
                            // display the Modify and Forget buttons.
                            @if context.saved_aps.contains(&selected) {
                                @if context.wlan_ssid != selected && ap.state == "Available" {
                                    form id="wifiConnect" action="/settings/network/wifi/connect" method="post" {
                                        // hidden element: allows ssid to be sent in request
                                        input id="connectSsid" name="ssid" type="text" value=(ssid) style="display: none;";
                                        input id="connectWifi" class="button button-primary center" title="Connect to Network" type="submit" value="Connect";
                                    }
                                }
                                a class="button button-primary center" href="/settings/network/wifi/modify/"{(ssid)}"" { "Modify" }
                                form id="wifiForget" action="/settings/network/wifi/forget" method="post" {
                                    // hidden element: allows ssid to be sent in request
                                    input id="forgetSsid" name="ssid" type="text" value=(ssid) style="display: none;";
                                    input id="forgetWifi" class="button button-warning center" title="Forget Network" type="submit" value="Forget";
                                }
                            }
                            @if !selected_is_saved {
                                // Display the Add button if AP creds not already in saved networks list
                                a class="button button-primary center" href="/settings/network/wifi/add/"{(ssid)}"" { "Add" };
                            }
                            a class="button button-secondary center" href="/settings/network/wifi" title="Cancel" { "Cancel" }
                        }
                        (templates::snippets::flash_message(flash_msg, flash_name))
                    }
                }
            }
            // TODO: improve the styling of this
            None => {
                div class="card center" {
                    p { "Selected access point was not found in-range or in the list of saved access points" }
                }
            }
        }
    };

    templates::base::base(back, title, content)
}

// /settings/network/wifi
pub fn list_aps() -> PreEscaped<String> {
    // retrieve network list data
    let context = NetworkListContext::build();

    let back = "/settings/network".to_string();
    let title = "WiFi Networks".to_string();

    let content = html! {
        (PreEscaped("<!-- NETWORK ACCESS POINT LIST -->"))
        div class="card center" {
            div class="center list-container" {
                ul class="list" {
                    @if context.ap_state == *"up" {
                        li class="list-item light-bg warning-border" { "Enable WiFi client mode to view saved and available networks." }
                    } @else if !context.wlan_networks.is_empty() {
                        @for (ssid, state) in context.wlan_networks {
                            li {
                                @if ssid == context.wlan_ssid {
                                    a class="list-item link primary-bg" href="/settings/network/wifi/ssid/"{(ssid)}"" {
                                        img id="netStatus" class="icon icon-active icon-medium list-icon" src="/static/icons/wifi.svg" alt="WiFi online";
                                        p class="list-text" { (context.wlan_ssid) }
                                        label class="label-small list-label font-gray" for="netStatus" title="Status" { "Connected" }
                                    }
                                } @else if state == "Available" {
                                    a class="list-item link light-bg" href="/settings/network/wifi/ssid/"{(ssid)}"" {
                                        img id="netStatus" class="icon icon-inactive icon-medium list-icon" src="/static/icons/wifi.svg" alt="WiFi offline";
                                        p class="list-text" { (ssid) }
                                        label class="label-small list-label font-gray" for="netStatus" title="Status" { (state) }
                                    }
                                } @else {
                                    a class="list-item link" href="/settings/network/wifi/ssid/"{(ssid)}"" {
                                        img id="netStatus" class="icon icon-inactive icon-medium list-icon" src="/static/icons/wifi.svg" alt="WiFi offline";
                                        p class="list-text" { (ssid) }
                                        label class="label-small list-label font-gray" for="netStatus" title="Status" { (state) }
                                    }
                                }
                            }
                        }
                    }
                    li class="list-item light-bg" { "No saved or available networks found." }
                }
            }
        }
    };

    templates::base::base(back, title, content)
}

// /settings/network/wifi
pub fn modify_ap(
    ssid: Option<String>,
    flash_msg: Option<String>,
    flash_name: Option<String>,
) -> PreEscaped<String> {
    let back = "/settings/network".to_string();
    let title = "Modify WiFi Network".to_string();

    let content = html! {
        (PreEscaped("<!-- NETWORK MODIFY AP PASSWORD FORM -->"))
        div class="card center" {
            div class="card-container" {
                form id="wifiModify" action="/settings/network/wifi/modify" method="post" {
                    (PreEscaped("<!-- input for network ssid -->"))
                    input id="ssid" name="ssid" class="center input" type="text" placeholder="SSID" title="Network name (SSID) for WiFi access point" value=[ssid] autofocus;
                    (PreEscaped("<!-- input for network password -->"))
                    input id="pass" name="pass" class="center input" type="password" placeholder="Password" title="Password for WiFi access point";
                    div id="buttonDiv" {
                        input id="savePassword" class="button button-primary center" title="Save" type="submit" value="Save";
                        a class="button button-secondary center" href="/settings/network" title="Cancel" { "Cancel" }
                    }
                }
                (templates::snippets::flash_message(flash_msg, flash_name))
            }
        }
    };

    templates::base::base(back, title, content)
}

// /settings/network/dns
pub fn configure_dns(flash_msg: Option<String>, flash_name: Option<String>) -> PreEscaped<String> {
    // retrieve dyndns-related data
    let context = ConfigureDNSContext::build();

    let back = "/settings/network".to_string();
    let title = "Configure DNS".to_string();

    let content = html! {
        (PreEscaped("<!-- CONFIGURE DNS FORM -->"))
        div class="card center" {
            div class="form-container" {
                @if context.enable_dyndns {
                    (PreEscaped("<!-- DYNDNS STATUS INDICATOR -->"))
                    div id="dyndns-status-indicator" class="stack capsule{% if is_dyndns_online %} success-border{% else %} warning-border{% endif %}" {
                        div class="stack" {
                            @if context.is_dyndns_online {
                                label class="label-small font-near-black" { "Dynamic DNS is currently online." }
                            } else {
                                label class="label-small font-near-black" { "Dynamic DNS is enabled but may be offline." }
                            }
                        }
                    }
                }
                form id="configureDNS" action="/settings/network/dns" method="post" {
                    div class="input-wrapper" {
                        (PreEscaped("<!-- input for externaldomain -->"))
                        label id="external_domain" class="label-small input-label font-near-black" {
                            label class="label-small input-label font-gray" for="external_domain" style="padding-top: 0.25rem;" { "External Domain (optional)" }
                            input id="external_domain" class="form-input" style="margin-bottom: 0;" name="external_domain" type="text" title="external domain" value=(context.external_domain);
                        }
                    }
                    div class="input-wrapper" {
                        div {
                            (PreEscaped("<!-- checkbox for dynds flag -->"))
                            label class="label-small input-label font-gray" { "Enable Dynamic DNS" }
                            input style="margin-left: 0px;" id="enable_dyndns" name="enable_dyndns" title="Activate dyndns" type="checkbox" { @if context.enable_dyndns { "checked" } };
                        }
                    }
                    div class="input-wrapper" {
                        (PreEscaped("<!-- input for dyndns -->"))
                        label id="cut" class="label-small input-label font-near-black" {
                            label class="label-small input-label font-gray" for="cut" style="padding-top: 0.25rem;" { "Dynamic DNS Domain" }
                            input id="dyndns_domain" class="alert-input" name="dynamic_domain" placeholder="" type="text" title="dyndns_domain" value=(context.dyndns_subdomain) { ".dyn.peachcloud.org" }
                        }
                    }
                    div id="buttonDiv" {
                        input id="configureDNSButton" class="button button-primary center" title="Add" type="submit" value="Save";
                    }
               }
            }
            (PreEscaped("<!-- FLASH MESSAGE -->"))
            (templates::snippets::flash_message(flash_msg, flash_name))
        }
    };

    templates::base::base(back, title, content)
}

// /settings/network
pub fn menu() -> PreEscaped<String> {
    let ap_state = network::ap_state();

    let back = "/settings".to_string();
    let title = "Network Settings".to_string();

    let content = html! {
        (PreEscaped("<!-- NETWORK SETTINGS CARD -->"))
        div class="card center" {
            (PreEscaped("<!-- BUTTONS -->"))
            div id="buttons" {
                a class="button button-primary center" href="/settings/network/wifi/add" title="Add WiFi Network" { "Add WiFi Network" }
                a id="configureDNS" class="button button-primary center" href="/settings/network/dns" title="Configure DNS" { "Configure DNS" }
                (PreEscaped("<!-- if ap is up, show \"Enable WiFi\" button, else show \"Deploy Access Point\" -->"))
                @if ap_state == *"up" {
                    a id="connectWifi" class="button button-primary center" href="/settings/network/wifi/activate" title="Enable WiFi" { "Enable WiFi" }
                } @else {
                    a id="deployAccessPoint" class="button button-primary center" href="/settings/network/ap/activate" title="Deploy Access Point" { "Deploy Access Point" }
                }
                a id="listWifi" class="button button-primary center" href="/settings/network/wifi" title="List WiFi Networks" { "List WiFi Networks" }
                a id="viewStatus" class="button button-primary center" href="/status/network" title="View Network Status" { "View Network Status" }
            }
        }
    };

    templates::base::base(back, title, content)
}
